/*
 * misc.h
 *
 *  Created on: Jul 27, 2015
 *      Author: dave
 */

#ifndef MISC_H_
#define MISC_H_

#include <type_traits>
#include <iostream>
#include <vector>
#include <algorithm>
#include <utility>

namespace tst
{

//! Visit function pointer type alias.
template <typename Iterator>
using visit_func = void (*)(Iterator first, Iterator last);


template <typename Iterator>
void visit_show(Iterator first, Iterator last)
{
	for (Iterator i = first; i != last; i++)
		std::cout << *i << " ";
	std::cout << std::endl;
}

template <typename Iterator>
void visit_null(Iterator first, Iterator last)
{
}

//! Rearranges the elements in the range [first, last) into
//! the next lexicographically greater permutation.
//!
//! @param first
//!		Sequence first element iterator
//! @param last
//!    	Sequence last element iterator
//! @return
//!		True if the function could rearrange the object as a
//!    	lexicographicaly greater permutation. Otherwise, the
//!    	function returns false to indicate that the arrangement
//!    	is not greater than the previous, but the lowest
//!    	possible (sorted in ascending order).
template <typename Iterator>
bool next_permute(Iterator first, Iterator last)
{
	Iterator &a = first;
	int n = last-first;

	// Step 2 : Find i
	int i = n-2;
	while (0 <= i && a[i] >= a[i+1])
		i--;

	if (i < 0)	// Termination condition
	{
		// rearrange to the smaller permutation in lex order
		for (int k = 0, j = n-1; k < j; k++, j--)
			std::swap(a[k], a[j]);
		return false;
	}

	// Step 3: increase a[i]
	int j = n-1;
	while (a[i] >= a[j])
		 j--;
	std::swap(a[i], a[j]);

	// Step 4: reverse a[i+1] ... a[n-1]
	for (int k = i+1, j = n-1; k < j; k++, j--)
		std::swap(a[k], a[j]);

	return true;
}

//! Generates and visits all the sequence permutations.
//!
//! @param first
//!		Sequence first element iterator.
//! @param last
//!     Sequence last element iterator.
//! @param visit
//!  	Visit function pointer.
template <typename Iterator>
void visit_permutations(Iterator first, Iterator last,
						visit_func<Iterator> visit)
{
	do
	{
		// Step 1 : Visit
		visit(first, last);
	} while (next_permute(first, last));
}

//! Generates the lexicographically smaller partition that can
//! be obtained from the integer n filling the sequence elements.
//! All but the last elements of the sequence are set to 1.
//! Precondition: (last-first) <= n
//!
//! @param n
//! 	Integer to partition.
//! @param first
//! 	Sequence first element iterator.
//! @param last
//!  	Sequence last element iterator.
template <typename Integer, typename Iterator>
void first_partition(Integer n, Iterator first, Iterator last)
{
	Iterator& a = first;
	Integer m = last-first;

	// Step 1 : initialize
	while (a != last-1)
		*a++ = 1;
	*a = n - m  + 1;
}

//! Uses the input sequence [first,last) elements to generate the
//! next greater partition in lexicographic order.
//!
//! @param first
//! 	Sequence first element iterator.
//! @param last
//!  	Sequence last element iterator.
//! @return
//! 	True if the function could generate a lexicographically
//! 	greater partition. Otherwise, the function returns false
//! 	to indicate that the arrangement is not greater than the
//! 	previous, but the lowest possible (in ascending order).
template <typename Iterator>
bool next_partition(Iterator first, Iterator last)
{
	Iterator &a = first;
	int m = last-first;

	if (m < 2)
		return false;

	if (a[m-2] < a[m-1]-1)
	{
		// Step 3 : tweak a[0] and a[1]
		a[m-2]++;
		a[m-1]--;
	}
	else
	{
		// Step 4 : find j
		int j = m-3;
		int s = a[m-2] + a[m-1] - 1;
		while (j != -1 && a[j] >= a[m-1]-1)
		{
			s += a[j];
			j--;
		}
		if (j == -1)	// Termination condition
		{
			// Reset to the smallest partition
			first_partition(s+1, first, last);
			return false;
		}

		// Step 5 : increase a[j]
		int x = a[j] + 1;
		a[j] = x;
		j++;

		// Step 6 : tweak a[j]...a[m-1]
		while (j < m-1)
		{
			a[j] = x;
			s -= x;
			j++;
		}
		a[j] = s;
	}

	return true;
}

//! Generates and visits all the sequence partitions.
//!
//! @param first
//!		Sequence first element iterator.
//! @param last
//!     Sequence last element iterator.
//! @param visit
//!  	Visit function pointer.
template <typename Iterator>
void visit_partitions(Iterator first, Iterator last,
					  visit_func<Iterator> visit)
{
	do
	{
		// Step 2 : Visit
		visit(first, last);
	} while (next_partition(first, last));
}


void g2dlp(int n1, int n2, int turns);

static inline long double fact(long double x)
{
    return (x == 0) ? 1 : x*fact(x-1);
}

static inline long double comb(long double n, long double k)
{
	return fact(n)/(fact(k)*fact(n-k));
}

static inline long double paths_count(int n1, int n2, int k)
{
	long double p;
	if (k%2 == 0)
	{
		p = comb(n1-1, k)*comb(n2-1,k-1)
		  + comb(n2-1, k)*comb(n1-1, k-1);
	}
	else
	{
		p = 2.0*comb(n1-1, k-1)*comb(n2-1, k-1);
	}
	return p;
}


}


#endif /* MISC_H_ */
